import tkinter as tk
import tkinter.filedialog as fd
import pandas as pd
import numpy as np
import pystan
import matplotlib.pyplot as plt
import seaborn as sb

def askinfilename(msg = None, types = [('', '*.*')]):
    """
            入力用ファイル名の設定
    """
    rt = tk.Tk()
    rt.withdraw()
    flnm = fd.askopenfilename(title = msg, filetypes = types)
    rt.destroy()
    return flnm

def askoutfilename(msg = None, types = [('', '*.*')], deftype = '.txt'):
    """
            出力用ファイル名の設定
    """
    rt = tk.Tk()
    rt.withdraw()
    flnm = fd.asksaveasfilename(title = msg, filetypes = types,
                                           defaultextension = deftype)
    rt.destroy()
    return flnm


finnm = askinfilename('入力データExcelファイル', types = [('Excel(*.xlsx)', '*.xlsx')])
foutnm = askoutfilename(msg = '画像保存ファイル', types = [('画像 (*.png)', '*.png')],
                       deftype = '.png')

xlsx_fl = pd.ExcelFile(finnm)
data_xlsx = pd.read_excel(xlsx_fl,  header = None)
data = data_xlsx.values

VarName1 = data[0, 1]
VarName2 = data[0, 2]
print(VarName1, VarName2)
X = data[1:, 1:3]
X = [[v[0], v[1]] for v in X]
print(X)

DataStan = {'N': len(X), 'X': X}

sm = pystan.StanModel(file = 'DifRelModel.stan')
fit = sm.sampling(data = DataStan, n_jobs = 1)
print(fit)

#       日本語表記の設定
plt.rcParams['font.family'] = ['Yu Gothic', 'Noto Sans CJK JP', 'Hiragino Maru Gothic Pro']

mu_025, mu_50, mu_975 = np.percentile(fit['mu'], [2.5, 50, 97.5])
plt.title(('差({0} - {1})の平均の事後分布'.format(VarName1, VarName2)) + \
          '\nMed = {0:.2f},  95%CI = [{1:.2f}, {2:.2f}]'.format(
              mu_50, mu_025, mu_975), fontsize = 20)
sb.kdeplot(fit['mu'])
plt.xlabel('{0:} - {1:}の平均値'.format(VarName1, VarName2), fontsize = 16)
plt.yticks([])
plt.tight_layout()
plt.savefig(foutnm)
plt.show()

sgm_025, sgm_50, sgm_975 = np.percentile(fit['sgm'], [2.5, 50, 97.5])
plt.title(('差({0} - {1})の標準偏差の事後分布'.format(VarName1, VarName2)) + \
          '\nMed = {0:.2f},  95%CI = [{1:.2f}, {2:.2f}]'.format(
              sgm_50, sgm_025, sgm_975), fontsize = 20)
sb.kdeplot(fit['sgm'])
plt.xlabel('{0:} - {1:}の標準偏差'.format(VarName1, VarName2), fontsize = 16)
plt.yticks([])
plt.tight_layout()
plt.savefig(foutnm[:-4] + '_sgm.png')
plt.show()
